{% load static %}
<html>
    <head>
        <title>
          Payment not found for {{ document.kind }} {{ document.series }}-{{ document.number }}
        </title>

        <style type="text/css">
            @page {
                size: a4;
                margin: 2cm;
            }
        </style>
        <link rel="stylesheet" type="text/css" href="{% static 'css/skeleton.css' %}">

        {% block extrahead %}
        {% endblock %}

    </head>

    <body>
        <div class="container">
            {% block content%}
                <h2>Transaction expired for</h2>

                {% block document %}
                    <div>
                        <h3>{{ document.kind }} {{ document.series }}-{{ document.number }}</h3>
                    </div>
                {% endblock %}
            {% endblock%}
        </div>
    </body>
</html>
