# Copyright (c) 2017 Presslabs SRL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

from django.forms import Form


class GenericTransactionForm(Form):
    def __init__(self, payment_method, transaction, request=None, *args, **kwargs):
        self.payment_method = payment_method
        self.transaction = transaction
        self.request = request

        super(GenericTransactionForm, self).__init__(*args, **kwargs)
