# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-06-25 08:55
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('silver', '0051_auto_20190603_0755'),
    ]

    operations = [
        migrations.AddField(
            model_name='billingdocumentbase',
            name='is_storno',
            field=models.BooleanField(default=False),
        ),
    ]
